"use strict";

var _tape = _interopRequireDefault(require("tape"));

var _reduxBundler = require("redux-bundler");

var _acFhirModels = require("ac-fhir-models");

var _fhirResources = require("./fhir-resources");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getStore = (delay = 0) => {
  const store = (0, _reduxBundler.composeBundlesRaw)(_fhirResources.fhirResourceBundle, {
    name: 'stubs',
    getExtraArgs: store => ({
      fhirFetch: () => new Promise(resolve => {
        setTimeout(() => resolve(store.fhirResponse), delay);
      })
    })
  })();
  return store;
};

(0, _tape.default)('fhirResourceBundle', async t => {
  const store = getStore();
  const practitionersRaw = store.selectPractitionersRaw();
  t.deepEqual(store.selectPractitionersRaw(), {}, 'Should be empty object to start');
  t.equal(store.selectPractitionersRaw(), store.selectPractitionersRaw(), 'Should return the same empty object to maximize memoization'); // prep and "fetch" response

  store.fhirResponse = _acFhirModels.appointmentDefinition.create({
    id: 'appointment_id'
  });
  await store.doFetchFhirResource('Appointment/appointment_id');
  t.equal(store.selectPractitionersRaw(), practitionersRaw, 'selectPractitionersRaw should not be affected by unrelated resource types');
  t.deepEqual(store.selectAppointmentsRaw(), {
    appointment_id: {
      resourceType: 'Appointment',
      id: 'appointment_id'
    }
  }, 'selectAppointmentsRaw should return items keyed by ID');
  t.equal(store.selectAppointmentsRaw(), store.selectAppointmentsRaw(), 'should return same appointments if called twice');
  const previousAppointments = store.selectAppointmentsRaw(); // prep and "fetch" response

  store.fhirResponse = _acFhirModels.patientDefinition.create({
    id: 'patient_id'
  });
  await store.doFetchFhirResource('Patient/patient_id');
  t.equal(previousAppointments, store.selectAppointmentsRaw(), 'should return same object if something else has changed');
  t.end();
});
(0, _tape.default)('fhirResourceBundle handles active operations', async t => {
  // prep and "fetch" response
  const store = getStore(100);
  store.fhirResponse = _acFhirModels.patientDefinition.create({
    id: 'patient_id',
    firstName: 'bob'
  });
  const promise = store.doFetchFhirResource('Patient/patient_id');
  t.deepEqual(store.selectPatientsRaw().patient_id, {
    resourceType: 'Patient',
    id: 'patient_id',
    activeOperation: 'load'
  }, 'should have "load" as active operation'); // let it finish

  await promise;
  t.deepEqual(store.selectPatientsRaw().patient_id, {
    resourceType: 'Patient',
    id: 'patient_id',
    name: [{
      given: ['bob']
    }]
  }, 'should now be complete and not have any active operation');
  t.end();
});
(0, _tape.default)('fhirResourceBundle handles receive resources appropriately', async t => {
  // prep and "fetch" response
  const store = getStore();
  const response = {
    entry: [{
      resource: _acFhirModels.appointmentDefinition.create({
        id: 'appointment_id_1'
      })
    }, {
      resource: _acFhirModels.appointmentDefinition.create({
        id: 'appointment_id_2'
      })
    }, {
      resource: _acFhirModels.appointmentDefinition.create({
        id: 'appointment_id_3'
      })
    }, {
      resource: _acFhirModels.appointmentDefinition.create({
        id: 'appointment_id_4'
      })
    }]
  };
  store.doReceiveFhirResource(response);
  t.deepEqual(store.selectAppointmentsRaw(), {
    appointment_id_1: {
      resourceType: 'Appointment',
      id: 'appointment_id_1'
    },
    appointment_id_2: {
      resourceType: 'Appointment',
      id: 'appointment_id_2'
    },
    appointment_id_3: {
      resourceType: 'Appointment',
      id: 'appointment_id_3'
    },
    appointment_id_4: {
      resourceType: 'Appointment',
      id: 'appointment_id_4'
    }
  }, 'should have received them all');
  t.end();
});
(0, _tape.default)('fhirResourceBundle.utilFindFhirResourceById()', async t => {
  // prep and "fetch" response
  const store = getStore();
  const response = {
    entry: [{
      resource: _acFhirModels.appointmentDefinition.create({
        id: 'appointment_id_1'
      })
    }, {
      resource: _acFhirModels.appointmentDefinition.create({
        id: 'appointment_id_2'
      })
    }]
  };
  t.deepEqual(store.utilFindFhirResourceById('appointment_id_1'), null, 'returns null if not found');
  store.doReceiveFhirResource(response);
  t.deepEqual(store.utilFindFhirResourceById('appointment_id_1'), {
    resourceType: 'Appointment',
    id: 'appointment_id_1'
  }, 'should be able to retrieve resource just by id');
  t.end();
});